@@ -99,9 +99,6 @@ Backend::~Backend()
 
 void Backend::play()
 {
-	if(alarm_playing)
-		return;
-
 	std::cout << "Backend::play()\n";
 	alarm_playing = true;
 
@@ -112,14 +109,12 @@ void Backend::play()
 
 void Backend::pause()
 {
-	if(!alarm_playing)
-		return;
-
 	std::cout << "Backend::pause()\n";
 	alarm_playing = false;
 
 	noise->pause();
-	stopVibrator();
+	if(use_vibration)
+		stopVibrator();
 }
 
 void Backend::volumeUp() { setVolume(volume + VOLUME_STEP); }
